@@ -1,5 +1,5 @@
 /* gobby - A GTKmm driven libobby client
- * Copyright (C) 2005 0x539 dev group
+ * Copyright (C) 2005, 2006 0x539 dev group
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public
@@ -19,9 +19,11 @@
 #ifndef _GOBBY_JOINPROGRESSDIALOG_HPP_
 #define _GOBBY_JOINPROGRESSDIALOG_HPP_
 
+#include <gtkmm/colorbutton.h>
 #include <obby/error.hpp>
-#include <obby/client_buffer.hpp>
+#include "buffer_def.hpp"
 #include "progressdialog.hpp"
+#include "colorsel.hpp"
 #include "config.hpp"
 
 namespace Gobby
@@ -30,7 +32,8 @@ namespace Gobby
 class JoinProgressDialog: public ProgressDialog
 {
 public:
-	JoinProgressDialog(Gtk::Window& parent, Config& config,
+	JoinProgressDialog(Gtk::Window& parent,
+	                   Config::ParentEntry& config_entry,
 	                   const Glib::ustring& hostname,
 	                   unsigned int port,
 	                   const Glib::ustring& username,
@@ -40,25 +43,89 @@ public:
 	 * JoinDialog will be reset to NULL after having transferred the data
 	 * to the caller.
 	 */
-	std::auto_ptr<obby::client_buffer> get_buffer();
+	std::auto_ptr<ClientBuffer> get_buffer();
 
 private:
-	typedef obby::client_buffer::connection_settings connection_settings;
+	typedef ClientBuffer::connection_settings connection_settings;
 
-	class Prompt: public DefaultDialog
+	class Prompt: public Gtk::Dialog
 	{
-	public:
+	protected:
 		Prompt(Gtk::Window& parent,
 		       const Glib::ustring& title,
 		       const Glib::ustring& info,
-		       Gtk::Widget& widget,
 		       const Gtk::StockID& icon);
-	protected:
+
+		void set_custom_widget(Widget& widget);
+
 		Gtk::Table m_table;
 		Gtk::Label m_info;
 		Gtk::Image m_icon;
 	};
 
+	class NamePrompt: public Prompt
+	{
+	public:
+		NamePrompt(Gtk::Window& parent,
+		           const Glib::ustring& initial_name);
+
+		Glib::ustring get_name() const;
+	protected:
+		void on_change();
+
+		const Glib::ustring m_initial_name;
+
+		Gtk::HBox m_box;
+		Gtk::Label m_label;
+		Gtk::Entry m_entry;
+	};
+
+	class ColorPrompt: public Prompt
+	{
+	public:
+		ColorPrompt(Gtk::Window& parent,
+		            Config::ParentEntry& config_entry,
+		            const Gdk::Color& initial_color);
+
+		Gdk::Color get_color() const;
+	protected:
+		ColorButton m_button;
+	};
+
+	class SessionPasswordPrompt: public Prompt
+	{
+	public:
+		SessionPasswordPrompt(Gtk::Window& parent);
+
+		Glib::ustring get_password() const;
+	protected:
+		void on_change();
+
+		Gtk::HBox m_box;
+		Gtk::Label m_label;
+		Gtk::Entry m_entry;
+	};
+
+	class UserPasswordPrompt: public Prompt
+	{
+	public:
+		UserPasswordPrompt(Gtk::Window& parent,
+		                   const Glib::ustring& initial_name);
+
+		Glib::ustring get_name() const;
+		Glib::ustring get_password() const;
+	protected:
+		void on_change();
+
+		const Glib::ustring m_initial_name;
+
+		Gtk::Table m_table;
+		Gtk::Label m_lbl_name;
+		Gtk::Label m_lbl_password;
+		Gtk::Entry m_ent_name;
+		Gtk::Entry m_ent_password;
+	};
+
 	virtual void on_thread(Thread& thread);
 	virtual void on_done();
 
@@ -78,7 +145,7 @@ private:
 
 	virtual void on_response(int response_id);
 
-	Config& m_config;
+	Config::ParentEntry& m_config_entry;
 
 	Glib::ustring m_hostname;
 	unsigned int m_port;
@@ -87,7 +154,7 @@ private:
 
 	Glib::ustring m_error;
 
-	std::auto_ptr<obby::client_buffer> m_buffer;
+	std::auto_ptr<ClientBuffer> m_buffer;
 
 	// Got done signal from connection thread
 	bool m_got_done;
@@ -95,6 +162,6 @@ private:
 	bool m_got_welcome;
 };
 
-}
+} // namespace Gobby
 
 #endif // _GOBBY_JOINPROGRESSDIALOG_HPP_
