@@ -122,19 +122,25 @@ namespace {
 	/** Callback function for std::list::sort to sort the languages by
 	 * their name.
 	 */
-	bool language_sort_callback(
-		const Glib::RefPtr<Gtk::SourceLanguage>& lang1,
-		const Glib::RefPtr<Gtk::SourceLanguage>& lang2
-	)
+	gint language_sort_callback(gconstpointer lang1, gconstpointer lang2)
 	{
-		return lang1->get_name() < lang2->get_name();
+		return strcmp(gtk_source_language_get_name(GTK_SOURCE_LANGUAGE(lang1)),
+			gtk_source_language_get_name(GTK_SOURCE_LANGUAGE(lang2)));
 	}
 }
 
 Gobby::Header::LanguageWrapper::LanguageWrapper(Action action,
-                                                Language language):
+                                                GtkSourceLanguage* language):
 	m_action(action), m_language(language)
 {
+	if(m_language != NULL)
+		g_object_ref(G_OBJECT(m_language));
+}
+
+Gobby::Header::LanguageWrapper::~LanguageWrapper()
+{
+	if(m_language != NULL)
+		g_object_unref(G_OBJECT(m_language));
 }
 
 Gobby::Header::LanguageWrapper::Action
@@ -143,7 +149,7 @@ Gobby::Header::LanguageWrapper::get_action() const
 	return m_action;
 }
 
-Gobby::Header::LanguageWrapper::Language
+GtkSourceLanguage*
 Gobby::Header::LanguageWrapper::get_language() const
 {
 	return m_language;
@@ -170,7 +176,9 @@ Gobby::Header::AutoAction::AutoAction(action_type action,
 
 void Gobby::Header::AutoAction::on_state_change(const ApplicationState& state)
 {
-	m_action->set_sensitive(state.query(m_inc_flags, m_exc_flags) );
+	bool sensitive = state.query(m_inc_flags, m_exc_flags);
+	m_action->set_sensitive(sensitive);
+	m_action->property_is_important().set_value(sensitive);
 }
 
 void Gobby::Header::AutoList::add(action_type action,
@@ -202,9 +210,8 @@ Gobby::Header::Error::Code Gobby::Header::Error::code() const
 	return static_cast<Code>(gobject_->code);
 }
 
-#include <iostream>
 Gobby::Header::Header(const ApplicationState& state,
-                      const LangManager& lang_mgr):
+                      GtkSourceLanguageManager* lang_mgr):
 	group_app(Gtk::ActionGroup::create("MenuApp") ),
 	group_session(Gtk::ActionGroup::create("MenuSession") ),
 	group_edit(Gtk::ActionGroup::create("MenuEdit") ),
@@ -601,19 +608,18 @@ Gobby::Header::Header(const ApplicationState& state,
 	group_help->add(action_help);
 	group_help->add(action_help_about);
 
-	// A kind of hack to ensure that
-	// Gtk::SourceLanguage::sourcelanguage_class_.init() is called.
-	// See the TODO item in Glib::wrap(GtkSourceLanguage*, bool) in
-	// sourcelanguage.cpp
-	//GtkSourceLanguage* lang = NULL;
-	//Glib::wrap(lang, false);
-
 	// Get available languages
-	std::list<Glib::RefPtr<Gtk::SourceLanguage> > lang_list =
-		lang_mgr->get_available_languages();
+#ifdef WITH_GTKSOURCEVIEW2
+	const GSList* list = gtk_source_language_manager_list_languages(
+		lang_mgr);
+#else
+	const GSList* list = gtk_source_languages_manager_get_available_languages(
+		lang_mgr);
+#endif
 
-	// Sort languages by name
-	lang_list.sort(&language_sort_callback);
+	// Copy the last, so we can sort languages by name
+	GSList* lang_list = g_slist_copy(const_cast<GSList*>(list));
+	lang_list = g_slist_sort(lang_list, &language_sort_callback);
 
 	// Add None-Language
 	Glib::RefPtr<Gtk::RadioAction> action = Gtk::RadioAction::create(
@@ -624,26 +630,20 @@ Gobby::Header::Header(const ApplicationState& state,
 	);
 
 	group_edit->add(action);
-	action_edit_syntax_languages.push_back(
-		LanguageWrapper(
-			action,
-			Glib::RefPtr<Gtk::SourceLanguage>(NULL)
-		)
-	);
+	action_edit_syntax_languages.push_back(LanguageWrapper(action, NULL));
 
 	// Add languages
-	for(std::list<Glib::RefPtr<Gtk::SourceLanguage> >::const_iterator iter =
-		lang_list.begin();
-	    iter != lang_list.end();
-	    ++ iter)
+	for(GSList* iter = lang_list; iter != NULL; iter = iter->next)
 	{
+		GtkSourceLanguage* language = GTK_SOURCE_LANGUAGE(iter->data);
+
 		// Get current language 
-		Glib::RefPtr<Gtk::SourceLanguage> language = *iter;
-		Glib::ustring language_xml_name = language->get_name();
+		Glib::ustring language_xml_name =
+			gtk_source_language_get_name(language);
 
 		// Build description string
 		obby::format_string str(_("Selects %0% as language") );
-		Glib::ustring name = language->get_name();
+		Glib::ustring name = language_xml_name;
 		str << name.raw();
 
 		// Add language to action group
@@ -651,7 +651,7 @@ Gobby::Header::Header(const ApplicationState& state,
 		action = Gtk::RadioAction::create(
 			m_lang_group,
 			"EditSyntaxLanguage" + language_xml_name,
-			language->get_name(),
+			name,
 			str.str()
 		);
 
@@ -676,6 +676,8 @@ Gobby::Header::Header(const ApplicationState& state,
 		m_ui_manager->add_ui_from_string(xml_desc);
 	}
 
+	g_slist_free(lang_list);
+
 	m_ui_manager->insert_action_group(group_app);
 	m_ui_manager->insert_action_group(group_session);
 	m_ui_manager->insert_action_group(group_edit);
@@ -718,19 +720,6 @@ Glib::RefPtr<const Gtk::AccelGroup> Gobby::Header::get_accel_group() const
 	return m_ui_manager->get_accel_group();
 }
 
-#if 0
-Glib::RefPtr<Gtk::SourceLanguagesManager> Gobby::Header::get_lang_manager()
-{
-	return m_lang_manager;
-}
-
-Glib::RefPtr<const Gtk::SourceLanguagesManager>
-Gobby::Header::get_lang_manager() const
-{
-	return m_lang_manager;
-}
-#endif
-
 Gtk::MenuBar& Gobby::Header::get_menubar()
 {
 	return *m_menubar;
