@@ -20,19 +20,21 @@
 #include "common.hpp"
 #include "hostdialog.hpp"
 
-Gobby::HostDialog::HostDialog(Gtk::Window& parent, Config& config):
-	DefaultDialog(_("Create obby session"), parent, true, true),
-	m_config(config),
+Gobby::HostDialog::HostDialog(Gtk::Window& parent,
+                              Config::ParentEntry& config_entry):
+	Gtk::Dialog(_("Create obby session"), parent, true, true),
+	m_config_entry(config_entry),
 	m_table(4, 2),
 	m_lbl_port(_("Port:"), Gtk::ALIGN_RIGHT),
 	m_lbl_name(_("Name:"), Gtk::ALIGN_RIGHT),
 	m_lbl_color(_("Colour:"), Gtk::ALIGN_RIGHT),
 	m_lbl_password(_("Password:"), Gtk::ALIGN_RIGHT),
 	m_lbl_session(_("Restore session:"), Gtk::ALIGN_RIGHT),
+	m_btn_color(config_entry),
 	m_ent_session(*this, _("Restore session") )
 {
 	m_ent_port.set_range(1024, 65535);
-	m_ent_port.set_value(config["session"]["host_port"].get(6522) );
+	m_ent_port.set_value(config_entry.get_value("host_port", 6522) );
 	m_ent_port.set_increments(1, 256);
 
 	m_ent_password.set_visibility(false);
@@ -43,14 +45,30 @@ Gobby::HostDialog::HostDialog(Gtk::Window& parent, Config& config):
 	default_color.set_green(0xcccc);
 	default_color.set_blue(0xffff);
 
-	Glib::ustring name =
-		config["session"]["name"].get(Glib::get_user_name() );
-	Gdk::Color color =
-		config["session"]["color"].get(Gdk::Color(default_color) );
+	Glib::ustring name = config_entry.get_value(
+		"name",
+		Glib::get_user_name()
+	);
+
+	Gdk::Color color = config_entry.get_value(
+		"color",
+		Gdk::Color(default_color)
+	);
 
 	m_ent_name.set_text(name);
 	m_btn_color.set_color(color);
 
+	sigc::slot<void> response(
+		sigc::bind(
+			sigc::mem_fun(*this, &Gtk::Dialog::response),
+			Gtk::RESPONSE_OK
+		)
+	);
+
+	m_ent_port.set_activates_default(true);
+	m_ent_name.set_activates_default(true);
+	m_ent_password.set_activates_default(true);
+
 	m_table.attach(m_lbl_port, 0, 1, 0, 1,
 		Gtk::SHRINK | Gtk::FILL, Gtk::SHRINK);
 	m_table.attach(m_lbl_name, 0, 1, 1, 2,
@@ -91,7 +109,15 @@ Gobby::HostDialog::HostDialog(Gtk::Window& parent, Config& config):
 	get_vbox()->pack_start(m_table);
 
 	add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
-	add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
+	Gtk::Button* host_btn = add_button(_("_Host"), Gtk::RESPONSE_OK);
+
+	Gtk::Image* img = Gtk::manage(
+		new Gtk::Image(Gtk::Stock::NETWORK, Gtk::ICON_SIZE_BUTTON)
+	);
+
+	host_btn->set_image(*img);
+
+	set_default_response(Gtk::RESPONSE_OK);
 
 	show_all();
 	set_border_width(10);
@@ -157,11 +183,10 @@ void Gobby::HostDialog::on_response(int response_id)
 	if(response_id == Gtk::RESPONSE_OK)
 	{
 		// Write new values into config
-		m_config["session"]["host_port"].set(get_port() );
-		m_config["session"]["name"].set(get_name() );
-		m_config["session"]["color"].set(get_color() );
+		m_config_entry.set_value("host_port", get_port() );
+		m_config_entry.set_value("name", get_name() );
+		m_config_entry.set_value("color", get_color() );
 	}
 
 	Gtk::Dialog::on_response(response_id);
 }
-
