@@ -27,10 +27,12 @@
 
 namespace
 {
-	bool lang_sort(const Glib::RefPtr<Gtk::SourceLanguage>& first,
-	               const Glib::RefPtr<Gtk::SourceLanguage>& second)
+	gint lang_sort(gconstpointer first, gconstpointer second)
 	{
-		return first->get_name() < second->get_name();
+		return strcmp(
+			gtk_source_language_get_name(GTK_SOURCE_LANGUAGE(first)),
+			gtk_source_language_get_name(GTK_SOURCE_LANGUAGE(second))
+			);
 	}
 
 	Gtk::ToolbarStyle rownum_to_toolstyle(int rownum)
@@ -39,6 +41,7 @@ namespace
 		{
 		case 0: return Gtk::TOOLBAR_TEXT;
 		case 1: return Gtk::TOOLBAR_ICONS;
+		case 3: return Gtk::TOOLBAR_BOTH_HORIZ;
 		case 2: default: return Gtk::TOOLBAR_BOTH;
 		}
 	}
@@ -214,20 +217,25 @@ Gobby::PreferencesDialog::Appearance::
 	Appearance(const Gobby::Preferences& preferences):
 	m_frame_toolbar(_("Toolbar") ),
 	m_frame_windows(_("Windows") ),
-	m_btn_remember(_("Remember the positions and states") )
+	m_btn_remember(_("Remember the positions and states") ),
+	m_btn_urgency_hint(
+	  _("Highlight the window on incoming chat messages") )
 {
 	Gtk::ToolbarStyle style = preferences.appearance.toolbar_show;
 	bool remember = preferences.appearance.remember;
+	bool urgency_hint = preferences.appearance.urgency_hint;
 
 	m_cmb_toolbar_style.append_text(_("Show text only") );
 	m_cmb_toolbar_style.append_text(_("Show icons only") );
 	m_cmb_toolbar_style.append_text(_("Show both icons and text") );
+	m_cmb_toolbar_style.append_text(_("Show text besides icons") );
 
 	switch(style)
 	{
 	case Gtk::TOOLBAR_TEXT: m_cmb_toolbar_style.set_active(0); break;
 	case Gtk::TOOLBAR_ICONS: m_cmb_toolbar_style.set_active(1); break;
 	case Gtk::TOOLBAR_BOTH: m_cmb_toolbar_style.set_active(2); break;
+	case Gtk::TOOLBAR_BOTH_HORIZ: m_cmb_toolbar_style.set_active(3); break;
 	default: break; // Avoids compiler warnings
 	}
 
@@ -240,7 +248,9 @@ Gobby::PreferencesDialog::Appearance::
 	m_box_windows.set_spacing(5);
 	m_box_windows.set_border_width(5);
 	m_btn_remember.set_active(remember);
+	m_btn_urgency_hint.set_active(urgency_hint);
 	m_box_windows.pack_start(m_btn_remember, Gtk::PACK_SHRINK);
+	m_box_windows.pack_start(m_btn_urgency_hint, Gtk::PACK_SHRINK);
 
 	m_frame_windows.add(m_box_windows);
 
@@ -259,6 +269,7 @@ void Gobby::PreferencesDialog::Appearance::
 	);
 
 	appearance.remember = m_btn_remember.get_active();
+	appearance.urgency_hint = m_btn_urgency_hint.get_active();
 }
 
 Gobby::PreferencesDialog::Font::Font(const Preferences& preferences):
@@ -287,6 +298,27 @@ void Gobby::PreferencesDialog::Font::set(Preferences::Font& font) const
 		font.desc = Pango::FontDescription(m_init_font);
 }
 
+Gobby::PreferencesDialog::Behaviour::Behaviour(const Preferences& preferences):
+	m_frame_documents(_("Document management") ),
+	m_btn_auto_open(_("Open new remotely-created documents automatically") )
+{
+	bool auto_open = preferences.behaviour.auto_open_new_documents;
+
+	m_btn_auto_open.set_active(auto_open);
+	m_box_documents.pack_start(m_btn_auto_open, Gtk::PACK_SHRINK);
+	m_frame_documents.add(m_box_documents);
+
+	m_box.pack_start(m_frame_documents, Gtk::PACK_SHRINK);
+
+	add(m_box);
+}
+
+void Gobby::PreferencesDialog::Behaviour::set(
+	Preferences::Behaviour& preferences) const
+{
+	preferences.auto_open_new_documents = m_btn_auto_open.get_active();
+}
+
 Gobby::PreferencesDialog::FileList::LanguageColumns::LanguageColumns()
 {
 	add(language);
@@ -302,7 +334,7 @@ Gobby::PreferencesDialog::FileList::FileColumns::FileColumns()
 
 Gobby::PreferencesDialog::FileList::FileList(Gtk::Window& parent,
                                              const Preferences& preferences,
-                                             const LangManager& lang_mgr):
+                                             GtkSourceLanguageManager* lang_mgr):
 	m_parent(parent), m_lang_mgr(lang_mgr),
 	m_viewcol_pattern(_("Pattern"), file_columns.pattern),
 	m_viewcol_lang(_("Language"), m_renderer_lang),
@@ -316,24 +348,29 @@ Gobby::PreferencesDialog::FileList::FileList(Gtk::Window& parent,
 	m_lang_list(Gtk::ListStore::create(lang_columns) ),
 	m_file_list(Gtk::ListStore::create(file_columns) )
 {
-	// Create language list
-	std::list<Glib::RefPtr<Gtk::SourceLanguage> > languages =
-		lang_mgr->get_available_languages();
+#ifdef WITH_GTKSOURCEVIEW2
+	const GSList* list =
+		gtk_source_language_manager_list_languages(lang_mgr);
+#else
+	const GSList* list =
+		gtk_source_languages_manager_get_available_languages(lang_mgr);
+#endif
 
-	languages.sort(&lang_sort);
+	GSList* languages = g_slist_copy(const_cast<GSList*>(list));
+	languages = g_slist_sort(languages, &lang_sort);
 
-	for(std::list<Glib::RefPtr<Gtk::SourceLanguage> >::const_iterator iter =
-		languages.begin();
-	    iter != languages.end();
-	    ++ iter)
+	for(GSList* iter = languages; iter != NULL; iter = iter->next)
 	{
 		Gtk::TreeIter tree_it = m_lang_list->append();
-		(*tree_it)[lang_columns.language] = *iter;
-		(*tree_it)[lang_columns.language_name] = (*iter)->get_name();
+		(*tree_it)[lang_columns.language] = GTK_SOURCE_LANGUAGE(iter->data);
+		(*tree_it)[lang_columns.language_name] =
+			gtk_source_language_get_name(GTK_SOURCE_LANGUAGE(iter->data));
 
-		m_lang_map[*iter] = tree_it;
+		m_lang_map[GTK_SOURCE_LANGUAGE(iter->data)] = tree_it;
 	}
 
+	g_slist_free(languages);
+
 	m_renderer_pattern = static_cast<Gtk::CellRendererText*>(
 		m_viewcol_pattern.get_first_cell_renderer()
 	);
@@ -376,8 +413,8 @@ Gobby::PreferencesDialog::FileList::FileList(Gtk::Window& parent,
 	    iter != filelist.end();
 	    ++ iter)
 	{
-		std::list<Glib::ustring> mime_types =
-			iter.language()->get_mime_types();
+/*		std::list<Glib::ustring> mime_types =
+			iter.language()->get_mime_types();*/
 
 		Gtk::TreeIter tree_it = m_file_list->append();
 		(*tree_it)[file_columns.pattern] = iter.pattern();
@@ -482,8 +519,32 @@ void Gobby::PreferencesDialog::FileList::
 	on_mimetype_edited(const Glib::ustring& path,
 	                   const Glib::ustring& new_text)
 {
-	Glib::RefPtr<Gtk::SourceLanguage> lang =
-		m_lang_mgr->get_language_from_mime_type(new_text);
+#ifdef WITH_GTKSOURCEVIEW2
+	const GSList* list =
+		gtk_source_language_manager_list_languages(m_lang_mgr);
+
+	GtkSourceLanguage* lang = NULL;
+	for(; list != NULL; list = list->next)
+	{
+		GtkSourceLanguage* language = GTK_SOURCE_LANGUAGE(list->data);
+		gchar** mime_types = gtk_source_language_get_mime_types(language);
+		for(gchar** mime_type = mime_types; *mime_type != NULL; ++ mime_type)
+		{
+			if(strcmp(*mime_type, new_text.c_str()) == 0)
+			{
+				lang = language;
+				break;
+			}
+		}
+
+		g_strfreev(mime_types);
+		if(lang != NULL) break;
+	}
+#else
+	GtkSourceLanguage* lang =
+		gtk_source_languages_manager_get_language_from_mime_type(
+			m_lang_mgr, new_text.c_str());
+#endif
 
 	if(!lang)
 	{
@@ -516,7 +577,7 @@ void Gobby::PreferencesDialog::FileList::
 {
 	// We do not get an iterator/path/whatever that points to the
 	// chosen language in the language list.
-	Glib::RefPtr<Gtk::SourceLanguage> lang(NULL);
+	GtkSourceLanguage* lang = NULL;
 	Gtk::TreeNodeChildren children = m_lang_list->children();
 
 	for(Gtk::TreeIter iter = children.begin();
@@ -589,7 +650,7 @@ void Gobby::PreferencesDialog::FileList::on_file_remove()
 }
 
 void Gobby::PreferencesDialog::FileList::set_language(const Gtk::TreeIter& row,
-                                                      const Language& lang)
+                                                      GtkSourceLanguage* lang)
 {
 	map_type::const_iterator lang_it = m_lang_map.find(lang);
 	if(lang_it == m_lang_map.end() )
@@ -600,20 +661,35 @@ void Gobby::PreferencesDialog::FileList::set_language(const Gtk::TreeIter& row,
 		);
 	}
 
-	std::list<Glib::ustring> mime_types = lang->get_mime_types();
-
 	(*row)[file_columns.language] = lang_it->second;
-	(*row)[file_columns.mime_type] = mime_types.front();
+#ifdef WITH_GTKSOURCEVIEW2
+	gchar** mime_types = gtk_source_language_get_mime_types(lang);
+
+	if(mime_types && *mime_types)
+		(*row)[file_columns.mime_type] = *mime_types;
+
+	g_strfreev(mime_types);
+#else
+	GSList* mime_types = gtk_source_language_get_mime_types(lang);
+	for(GSList* cur = mime_types; cur != NULL; cur = cur->next)
+	{
+		if(cur == mime_types)
+			(*row)[file_columns.mime_type] =
+				static_cast<gchar*>(cur->data);
+		g_free(cur->data);
+	}
+	g_slist_free(mime_types);
+#endif
 }
 
 Gobby::PreferencesDialog::PreferencesDialog(Gtk::Window& parent,
                                             const Preferences& preferences,
-                                            const LangManager& lang_mgr,
+                                            GtkSourceLanguageManager* lang_mgr,
                                             bool local)
  : Gtk::Dialog(_("Preferences"), parent, true),
    m_page_editor(preferences, m_tooltips), m_page_view(preferences),
    m_page_appearance(preferences), m_page_font(preferences),
-   m_page_files(*this, preferences, lang_mgr)
+   m_page_behaviour(preferences), m_page_files(*this, preferences, lang_mgr)
 {
 	m_notebook.append_page(m_page_editor, _("Editor") );
 	m_notebook.append_page(m_page_view, _("View") );
@@ -621,6 +697,7 @@ Gobby::PreferencesDialog::PreferencesDialog(Gtk::Window& parent,
 	// Appearance only affects the global Gobby window
 	if(!local) m_notebook.append_page(m_page_appearance, _("Appearance") );
 	m_notebook.append_page(m_page_font, _("Font") );
+	if(!local) m_notebook.append_page(m_page_behaviour, _("Behaviour") );
 	if(!local) m_notebook.append_page(m_page_files, _("Files") );
 
 	get_vbox()->set_spacing(5);
@@ -642,5 +719,7 @@ void Gobby::PreferencesDialog::set(Preferences& preferences) const
 	m_page_view.set(preferences.view);
 	m_page_appearance.set(preferences.appearance);
 	m_page_font.set(preferences.font);
+	m_page_behaviour.set(preferences.behaviour);
 	m_page_files.set(preferences.files);
 }
+
