@@ -47,17 +47,20 @@
 
 Gobby::Window::Window(const IconManager& icon_mgr, Config& config):
 	Gtk::Window(Gtk::WINDOW_TOPLEVEL), m_config(config),
-	m_lang_manager(Gtk::SourceLanguagesManager::create() ),
-	m_preferences(m_config, m_lang_manager), m_icon_mgr(icon_mgr),
-#ifdef WITH_ZEROCONF
-	m_zeroconf(NULL),
+#ifdef WITH_GTKSOURCEVIEW2
+	m_lang_manager(gtk_source_language_manager_new()),
+#else
+	m_lang_manager(gtk_source_languages_manager_new()),
 #endif
+	m_preferences(m_config, m_lang_manager), m_icon_mgr(icon_mgr),
 	m_application_state(APPLICATION_NONE),
 	m_document_settings(*this),
 	m_header(m_application_state, m_lang_manager),
+	m_folder(m_header, m_preferences),
 	m_userlist(
 		*this,
 		m_header,
+		m_folder,
 		m_preferences,
 		config.get_root()["windows"]
 	),
@@ -65,12 +68,15 @@ Gobby::Window::Window(const IconManager& icon_mgr, Config& config):
 		*this,
 		m_document_settings,
 		m_header,
+		m_folder,
 		m_preferences,
 		config.get_root()["windows"]
 	),
-	m_finddialog(*this), m_gotodialog(*this),
-	m_folder(m_header, m_preferences), m_chat(*this),
+	m_chat(*this, m_preferences),
 	m_statusbar(m_header, m_folder)
+#ifdef WITH_AVAHI
+	,m_glib_poll(avahi_glib_poll_new(NULL, G_PRIORITY_DEFAULT))
+#endif
 {
 	// Header
 	m_header.action_app_session_create->signal_activate().connect(
@@ -166,7 +172,7 @@ Gobby::Window::Window(const IconManager& icon_mgr, Config& config):
 	m_application_state.modify(APPLICATION_INITIAL, APPLICATION_NONE);
 
 	show_all_children();
-	if(!show_chat) m_frame_chat.hide();
+	if(!show_chat) m_frame_chat.hide_all();
 
 	set_title("Gobby");
 	set_default_size(640, 480);
@@ -175,14 +181,21 @@ Gobby::Window::Window(const IconManager& icon_mgr, Config& config):
 	// Initialise Zeroconf
 	try
 	{
+#ifdef WITH_AVAHI
+		m_zeroconf.reset(new obby::zeroconf_avahi(avahi_glib_poll_get(m_glib_poll)));
+#else
 		m_zeroconf.reset(new obby::zeroconf);
+		// Periodically check for events when not using Avahi Glib Poll
+		Glib::signal_timeout().connect(sigc::bind(sigc::mem_fun(*m_zeroconf.get(), &zeroconf_base::select), 0), 1500);
+#endif
 	}
 	catch(std::runtime_error&)
 	{
-		display_error(_("Howl initialisation failed. Probably you need "
-			"to run mDNSResponder as root prior to Gobby. "
-			"Zeroconf support is deactivated for this session."),
-			Gtk::MESSAGE_WARNING);
+		std::cerr << _("Zeroconf initialisation failed. Probably you "
+			"need to run avahi-daemon or mDNSResponder, depending "
+			"on the library you use, as root prior to Gobby. "
+			"Zeroconf support is deactivated for this session.");
+		std::cerr << std::endl;
 		m_zeroconf.reset();
 	}
 #endif
@@ -192,8 +205,8 @@ Gobby::Window::Window(const IconManager& icon_mgr, Config& config):
 		Config::ParentEntry& screen = config.get_root()["screen"];
 
 		// Restore the window's position from the configuration
-        	const int x = windows["main"].get_value<int>("x", 0);
-	        const int y = windows["main"].get_value<int>("y", 0);
+		const int x = windows["main"].get_value<int>("x", 0);
+		const int y = windows["main"].get_value<int>("y", 0);
 		const int w = windows["main"].get_value<int>("width", 0);
 		const int h = windows["main"].get_value<int>("height", 0);
 
@@ -241,6 +254,16 @@ Gobby::Window::~Window()
 		screen.set_value("width", scr->get_width() );
 		screen.set_value("height", scr->get_height() );
 	}
+
+	/* Free explictely to make sure that the avahi poll is no longer
+	 * referenced when we free it */
+#ifdef WITH_ZEROCONF
+	m_zeroconf.reset(NULL);
+#endif
+
+#ifdef WITH_AVAHI
+	avahi_glib_poll_free(m_glib_poll);
+#endif
 }
 
 bool Gobby::Window::on_delete_event(GdkEventAny* event)
@@ -489,7 +512,7 @@ void Gobby::Window::on_about()
 	dlg.set_comments(_("A collaborative text editor"));
 
 	dlg.set_copyright(
-		"Copyright (C) 2005-2006 0x539 dev group <crew@0x539.de>"
+		"Copyright (C) 2005-2007 0x539 dev group <crew@0x539.de>"
 	);
 
 	dlg.set_logo(m_icon_mgr.gobby);
@@ -547,9 +570,19 @@ void Gobby::Window::on_about()
 
 void Gobby::Window::on_folder_document_add(DocWindow& window)
 {
-	// Select newly created page
-	m_folder.set_current_page(m_folder.page_num(window) );
-	window.grab_focus();
+	// Select newly created page if not automatically opened
+	if(!m_document_settings.get_automatically_opened(window.get_info() ))
+	{
+		m_folder.set_current_page(m_folder.page_num(window) );
+		window.grab_focus();
+	}
+
+	// Unset modifified flag when locally opened
+	if(!m_local_file_path.empty())
+	{
+		gtk_text_buffer_set_modified(
+			GTK_TEXT_BUFFER(window.get_document().get_buffer()), FALSE);
+	}
 
 	if(m_folder.get_n_pages() == 1)
 	{
@@ -558,6 +591,12 @@ void Gobby::Window::on_folder_document_add(DocWindow& window)
 			APPLICATION_DOCUMENT,
 			APPLICATION_NONE
 		);
+
+		m_folder.set_show_tabs(false);
+	}
+	else
+	{
+		m_folder.set_show_tabs(true);
 	}
 }
 
@@ -574,6 +613,10 @@ void Gobby::Window::on_folder_document_remove(DocWindow& window)
 			APPLICATION_DOCUMENT
 		);
 	}
+	else if(m_folder.get_n_pages() == 1)
+	{
+		m_folder.set_show_tabs(false);
+	}
 }
 
 void Gobby::Window::on_folder_document_close_request(DocWindow& window)
@@ -587,8 +630,16 @@ void Gobby::Window::on_folder_tab_switched(DocWindow& window)
 	update_title_bar();
 }
 
-void Gobby::Window::on_settings_document_insert(const LocalDocumentInfo& info)
+void Gobby::Window::on_settings_document_insert(LocalDocumentInfo& info)
 {
+	// Mark automatically opened documents and subscribe to them.
+	if(m_preferences.behaviour.auto_open_new_documents
+		&& !info.is_subscribed() )
+	{
+		m_document_settings.set_automatically_opened(info, true);
+		info.subscribe();
+	}
+
 	// Set the path from which this document was opened,
 	// if we opened that file.
 	if(info.get_owner() == &m_buffer->get_self() &&
@@ -604,10 +655,6 @@ void Gobby::Window::on_settings_document_insert(const LocalDocumentInfo& info)
 			info,
 			m_local_encoding
 		);
-
-		// Clear local path
-		m_local_file_path.clear();
-		m_local_encoding.clear();
 	}
 	else
 	{
@@ -629,6 +676,10 @@ void Gobby::Window::on_document_create()
 		m_local_encoding = "UTF-8";
 		// Create new document
 		m_buffer->document_create(dlg.get_text(), "UTF-8", "");
+
+		// Clear local path
+		m_local_file_path.clear();
+		m_local_encoding.clear();
 	}
 }
 
@@ -674,6 +725,11 @@ void Gobby::Window::on_document_open()
 }
 
 void Gobby::Window::on_document_save()
+{
+	handle_document_save();
+}
+
+bool Gobby::Window::handle_document_save()
 {
 	// Get page
 	DocWindow* doc = get_current_document();
@@ -697,15 +753,21 @@ void Gobby::Window::on_document_save()
 				doc->get_info()
 			)
 		);
+		return true;
 	}
 	else
 	{
 		// Open save as dialog otherwise
-		on_document_save_as();
+		return handle_document_save_as();
 	}
 }
 
 void Gobby::Window::on_document_save_as()
+{
+	handle_document_save_as();
+}
+
+bool Gobby::Window::handle_document_save_as()
 {
 	// Get page
 	DocWindow* doc = get_current_document();
@@ -753,7 +815,8 @@ void Gobby::Window::on_document_save_as()
 
 	// Add buttons to close the dialog
 	dlg.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
-	dlg.add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
+	dlg.add_button(Gtk::Stock::SAVE, Gtk::RESPONSE_OK);
+	dlg.set_default_response(Gtk::RESPONSE_OK);
 
 	if(dlg.run() == Gtk::RESPONSE_OK)
 	{
@@ -765,7 +828,9 @@ void Gobby::Window::on_document_save_as()
 			dlg.get_filename(),
 			dlg.get_selector().get_encoding()
 		);
+		return true;
 	}
+	return false;
 }
 
 void Gobby::Window::on_document_close()
@@ -778,22 +843,28 @@ void Gobby::Window::on_document_close()
 
 void Gobby::Window::on_edit_search()
 {
-	m_finddialog.set_search_only(true);
-	m_finddialog.show();
-	m_finddialog.grab_focus();
+	if(m_finddialog.get() == NULL)
+		m_finddialog.reset(new FindDialog(*this));
+
+	m_finddialog->set_search_only(true);
+	m_finddialog->present();
 }
 
 void Gobby::Window::on_edit_search_replace()
 {
-	m_finddialog.set_search_only(false);
-	m_finddialog.show();
-	m_finddialog.grab_focus();
+	if(m_finddialog.get() == NULL)
+		m_finddialog.reset(new FindDialog(*this));
+
+	m_finddialog->set_search_only(false);
+	m_finddialog->present();
 }
 
 void Gobby::Window::on_edit_goto_line()
 {
-	m_gotodialog.show();
-	m_gotodialog.grab_focus();
+	if(m_gotodialog.get() == NULL)
+		m_gotodialog.reset(new GotoDialog(*this));
+
+	m_gotodialog->present();
 }
 
 void Gobby::Window::on_edit_preferences()
@@ -940,7 +1011,7 @@ void Gobby::Window::on_view_preferences()
 }
 
 void
-Gobby::Window::on_view_language(const Glib::RefPtr<Gtk::SourceLanguage>& lang)
+Gobby::Window::on_view_language(GtkSourceLanguage* language)
 {
 	// Set language of current document
 	DocWindow* doc = get_current_document();
@@ -952,18 +1023,18 @@ Gobby::Window::on_view_language(const Glib::RefPtr<Gtk::SourceLanguage>& lang)
 		);
 	}
 
-	doc->set_language(lang);
+	doc->set_language(language);
 }
 
 void Gobby::Window::on_window_chat()
 {
 	if(m_header.action_window_chat->get_active() )
 	{
-		m_frame_chat.show();
+		m_frame_chat.show_all();
 	}
 	else
 	{
-		m_frame_chat.hide();
+		m_frame_chat.hide_all();
 	}
 }
 
@@ -1367,6 +1438,10 @@ void Gobby::Window::open_local_file(const Glib::ustring& file,
 		m_buffer->document_create(
 			Glib::path_get_basename(file), "UTF-8", utf8_content
 		);
+
+		// Clear local path
+		m_local_file_path.clear();
+		m_local_encoding.clear();
 	}
 	catch(Glib::Exception& e)
 	{
@@ -1379,19 +1454,10 @@ void Gobby::Window::save_local_file(DocWindow& doc,
                                     const Glib::ustring& file,
                                     const std::string& encoding)
 {
-	// Open stream to file
-	std::ofstream stream(file.c_str() );
-
 	try
 	{
-		if(!stream)
-		{
-			obby::format_string str(
-				_("Could not open file '%0%' for writing")
-			);
-			str << file;
-			throw std::runtime_error(str.str() );
-		}
+		Glib::RefPtr<Glib::IOChannel> channel = Glib::IOChannel::create_from_file(file, "w");
+		channel->set_encoding("");
 
 		// Save content into file
 		std::string conv_content = doc.get_content().raw();
@@ -1404,8 +1470,8 @@ void Gobby::Window::save_local_file(DocWindow& doc,
 			);
 		}
 
-		stream << conv_content;
-		stream.close();
+		channel->write(conv_content);
+		channel->close();
 
 		m_document_settings.set_path(doc.get_info(), file);
 
@@ -1417,9 +1483,11 @@ void Gobby::Window::save_local_file(DocWindow& doc,
 		// Update title bar according to new path
 		update_title_bar();
 		// Unset modifified flag
-		doc.get_document().get_buffer()->set_modified(false);
+		gtk_text_buffer_set_modified(
+			GTK_TEXT_BUFFER(doc.get_document().get_buffer()),
+			FALSE);
 	}
-	catch(Glib::ConvertError& e)
+	catch(Glib::Error& e)
 	{
 		display_error(e.what() );
 	}
@@ -1477,6 +1545,9 @@ void Gobby::Window::close_document(DocWindow& window)
 
 		// Show the dialog
 		int result = dlg.run();
+		// Hide it because we cannot back up to it if a later dialog
+		// is cancelled.
+		dlg.hide();
 
 		switch(result)
 		{
@@ -1486,9 +1557,8 @@ void Gobby::Window::close_document(DocWindow& window)
 		case Gtk::RESPONSE_ACCEPT:
 			/* Save the document before closing it */
 			m_folder.set_current_page(m_folder.page_num(window) );
-			// TODO: Do not close the document if the user cancells
-			// the save dialog.
-			on_document_save();
+			if(!handle_document_save() )
+				return;
 			break;
 		case Gtk::RESPONSE_CANCEL:
 		case Gtk::RESPONSE_DELETE_EVENT:
